@@ -2,10 +2,12 @@ package V;
 use strict;
 
 use vars qw( $VERSION $NO_EXIT );
-$VERSION  = "0.16_03";
+$VERSION  = "0.17_00";
 
 $NO_EXIT ||= 0; # prevent import() from exit()ing and fall of the edge
 
+our $DEBUG ||= 0;
+
 =head1 NAME
 
 V - Print version of the specified module(s).
@@ -24,13 +26,25 @@ modules without loading them:
     require V;
     printf "%s has version '%s'\n", "V", V::get_version( "V" );
 
+Starting with version B<0.17>, V will show all C<package>s or C<class>es in a file.
+
 If you want all available files/versions from C<@INC>:
 
     require V;
     my @all_V = V::Module::Info->all_installed("V");
     printf "%s:\n", $all_V[0]->name;
-    printf "\t%-50s - %s\n", $_->file, $_->version
-        for @all_V;
+    for my $file (@all_V) {
+        my ($versions) = $file->version; # Must be list context
+        if (@$versions > 1) {
+            printf "\t%s:\n", $file->name;
+            for my $ver (@versions) {
+                print "\t    %-30s: %s\n", $ver->{pkg}, $ver->{version};
+            }
+        }
+        else {
+            printf "\t%-50s - %s\n", $file->file, $versions->[0]{version};
+        }
+    }
 
 Each element in that array isa C<V::Module::Info> object with 3 attributes and a method:
 
@@ -51,8 +65,36 @@ The base directory (from C<@INC>) where the package-file was found.
 =item I<method> B<version>
 
 This method will look through the file to see if it can find a version
-assignment in the file and uses that determine the version. As of version
-0.13_01, all versions found are passed through the L<version> module.
+assignment in the file and uses that to determine the version. As of version
+B<0.13_01>, all versions found are passed through the L<version> module.
+
+As of version B<0.16_03> we look for all types of version declaration:
+
+    package Foo;
+    our $VERSION = 0.42;
+
+and
+
+    package Foo 0.42;
+
+and
+
+    package Foo 0.42 { ... }
+
+Not only do we look for the C<package> keyword, but also for C<class>.
+In list context this method will return an arrayref to a list of structures:
+
+=over 8
+
+=item I<pkg>
+
+The name of the C<package>/C<class>.
+
+=item I<version>
+
+The version for that C<package>/C<class>.
+
+=back
 
 =back
 
@@ -207,6 +249,7 @@ sub all_installed {
         }
     }
 
+    do {print {*STDERR} "# $file: @{[scalar $_->version]}\n" for @modules} if $V::DEBUG;
     return @modules;
 }
 
@@ -236,7 +279,7 @@ sub version {
 
         next if $cur_pkg =~ m{^V::Module::Info};
 
-        if (m/^[^=]*([\$*])(([\w\:\']*)\bVERSION)\b.*\=(?!~)/) {
+        if (m/(?:our)?\s*([\$*])(([\w\:\']*)\bVERSION)\s*\=(?![=~])/) {
             { local($1, $2); ($_ = $_) = m/(.*)/; } # untaint
             my ($sigil, $name) = ($1, $2);
             next if m/\$$name\s*=\s*eval.+\$$name/;
@@ -251,7 +294,7 @@ sub version {
             };
         }
         # perl 5.12.0+
-        elsif (m/^\s* (?:package|class) \s+ [^\s]+ \s+ ([^;\{]+) [;\{]/x) {
+        elsif (m/^\s* (?:package|class) \s+ [^\s]+ \s+ (v?[0-9.]+) \s* [;\{]/x) {
             my $ver = $1;
             if ( $] >= 5.012000 ) {
                 $eval{$cur_pkg}{prg} = qq{
@@ -275,6 +318,7 @@ sub version {
     while (my ($pkg, $dat) = each(%eval)) {
         my $result;
         if ($dat->{prg}) {
+            print {*STDERR} "# $pkg: $dat->{prg}\n" if $V::DEBUG;
             local $^W = 0;
             $result = eval($dat->{prg});
             warn "Could not eval '$dat->{prg}' in $parsefile: $@" if $@;
